==> woodpecker-debug-3.10.0-1-x86_64.pkg.tar.zst: usr/src/debug/woodpecker/woodpecker/server/forge/refresh.go <==
// Copyright 2023 Woodpecker Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package forge

import (
	"context"
	"time"

	"github.com/rs/zerolog/log"

	"go.woodpecker-ci.org/woodpecker/v3/server/model"
	"go.woodpecker-ci.org/woodpecker/v3/server/store"
)

// Refresher refreshes an oauth token and expiration for the given user. It
// returns true if the token was refreshed, false if the token was not refreshed,
// and error if it failed to refresh.
type Refresher interface {
	Refresh(context.Context, *model.User) (bool, error)
}

func Refresh(c context.Context, forge Forge, _store store.Store, user *model.User) {
	// Remaining ttl of 30 minutes (1800 seconds) until a token is refreshed.
	const tokenMinTTL = 1800

	if refresher, ok := forge.(Refresher); ok {
		// Check to see if the user token is expired or
		// will expire within the next 30 minutes (1800 seconds).
		// If not, there is nothing we really need to do here.
		if time.Now().UTC().Unix() < (user.Expiry - tokenMinTTL) {
			return
		}

		ok, err := refresher.Refresh(c, user)
		if err != nil {
			log.Error().Err(err).Msgf("refresh oauth token of user '%s' failed", user.Login)
		} else if ok {
			if err := _store.UpdateUser(user); err != nil {
				log.Error().Err(err).Msg("fail to save user to store after refresh oauth token")
			}
		}
	}
}

==> woodpecker-debug-3.11.0-1-x86_64.pkg.tar.zst: usr/src/debug/woodpecker/woodpecker/server/forge/refresh.go <==
// Copyright 2023 Woodpecker Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package forge

import (
	"context"
	"time"

	"github.com/rs/zerolog/log"

	"go.woodpecker-ci.org/woodpecker/v3/server/model"
	"go.woodpecker-ci.org/woodpecker/v3/server/store"
)

// Refresher is an optional interface for OAuth token refresh support.
//
// Tokens are checked before each operation. If expiring within 30 minutes,
// Refresh() is called automatically.
//
// Implementations: GitLab, Bitbucket (GitHub/Gitea tokens don't expire).
type Refresher interface {
	// Refresh attempts to refresh the user's OAuth access token.
	// Should update u.AccessToken, u.RefreshToken, and u.Expiry.
	// Returns true if any fields were updated.
	// Caller must persist updated user to database.
	Refresh(ctx context.Context, u *model.User) (bool, error)
}

func Refresh(c context.Context, forge Forge, _store store.Store, user *model.User) {
	// Remaining ttl of 30 minutes (1800 seconds) until a token is refreshed.
	const tokenMinTTL = 1800

	if refresher, ok := forge.(Refresher); ok {
		// Check to see if the user token is expired or
		// will expire within the next 30 minutes (1800 seconds).
		// If not, there is nothing we really need to do here.
		if time.Now().UTC().Unix() < (user.Expiry - tokenMinTTL) {
			return
		}

		ok, err := refresher.Refresh(c, user)
		if err != nil {
			log.Error().Err(err).Msgf("refresh oauth token of user '%s' failed", user.Login)
		} else if ok {
			if err := _store.UpdateUser(user); err != nil {
				log.Error().Err(err).Msg("fail to save user to store after refresh oauth token")
			}
		}
	}
}
